package cam.symfun;
/**
An instance of class SymbolicFunction implements a double valued function 
(of an arbitrary number of double arguments) as specified by an initialization 
character string (i.e. symbolic initialization). <P>
The syntax for the initialization string is that of general Java expressions. 
The initialization string can contain references to Java math functions 
(those in java.math) as well as symbolic constants. 
The method evaluate(...) is used to evaluate a SymbolicFuntion instance.
<P>
<A HREF="Samples.html">Samples</A>
<P>
@author Chris Anderson (C) UCLA 
@version 3/30/98
*/
public class SymbolicFunction
{ 
//
//######################################################################
//                     Class SymbolicFunction
//######################################################################
//   
// Created from CAMsymbolicFunction  -> Migrated C++ code to Java 3/30/98
// Chris Andersion (C) UCLA
//
//######################################################################
//
//######################################################################
//
//

/**
Creates a SymbolicFunction instance; use initialize(...) to specify the function. 
*/
public SymbolicFunction()
{
    opLib  = new RealOperatorLib();    
};

/**
Initializes the SymbolicFunction instance to a function of a single variable x.
@param expressionString A java.lang.String specifying the function in the variable x 
<br> e.g. <i>x^2 + 3.0 + cos(x)</i>.
@exception java.lang.Exception If an exeception is thrown, the detail message
of the exception (obtained with getMessage()) returns a string indicating 
the source of the initialization error. 
*/
public int  initialize(String expressionString) throws Exception 
{
    String [] V       = null;
    String [] C       = null;
    double [] Cvalues = null;
    
    V    = new String[1];
    V[0] = "x";
    return initialize(V,C,Cvalues,expressionString); 
};
/**
Initializes the SymbolicFunction instance to a function of n variables. 
@param   V. An array of java.lang.String of length n specifying the independent variables 
<br> e.g. for a function of two variables x and y V[0] = "x"; V[1] ="y";.
@param expressionString A java.lang.String specifying the function in the independent variables.
@exception java.lang.Exception If an exeception is thrown, the detail message
of the exception (obtained with getMessage()) returns a string indicating 
the source of the initialization error. 
*/
public int  initialize(String [] V, String expressionString) throws Exception 
{
    String [] C       = null;
    double [] Cvalues = null;
    
    return initialize(V,C,Cvalues,expressionString);  
};
/**
Initializes the SymbolicFunction instance to a function of n variables with m symbolic constants, e.g. 
the function specified as a*x^2 + b*x + c is a function of one variable, x,and three symboic constants
"a","b" and "c".
@param   V. An array of java.lang.String of length n specifying the independent variables 
@param   C. An array of java.lang.String of length m specifying the symbolic constants
@param   Cvalues. An array of java.lang.double of length m specifying the initial values for the 
symbolic constants
@param expressionString A java.lang.String specifying the function 
in the independent variables and 
symbolic constants.
@exception java.lang.Exception If an exeception is thrown, the detail message
of the exception (obtained with getMessage()) returns a string indicating 
the source of the initialization error. 
*/
public int  initialize(String [] V, String [] C, double [] Cvalues, String expressionString)
throws Exception 
{
     
    int i; int j;
    Vcount = 0; 
    Ccount = 0;
  
    constructorString = new String(expressionString);

    if(V != null)
    {
    Vcount = V.length;
    variableNames = new String[Vcount];
    for(i = 0; i < Vcount; i++) variableNames[i] = new String(V[i]);
    }
    
    if(C != null)
    {
    Ccount = C.length;
    constantNames  = new String[Ccount];
    constantValues = new double[Ccount];
    for(i = 0; i < Ccount; i++) 
    {
        constantNames[i]  = new String(C[i]); 
        constantValues[i] = Cvalues[i];
    }
    }
    
    ExpressionTransform T = new ExpressionTransform();
    T.createTransform(V,C,constructorString);
    

    executionArray   = T.getExecutionArray();
    evaluationData   = new double[T.getEvaluationDataSize()];
    String [] sNames = T.getSymbolicNames();
    int Scount       = sNames.length;
 
//
//  Initialize Evaluation Data
//
    Double D;
    
    if(Vcount != 0)
    for(i = 0; i < Vcount; i++) evaluationData[i] = 0.0;

    for(i = Vcount,j = 0; i < Vcount + Ccount; i++,j++)
    evaluationData[i] = constantValues[j];

    for(i = Vcount + Ccount; i < Scount; i++)
    {
    D = Double.valueOf(sNames[i]);
    evaluationData[i] =D.doubleValue();
    }
    return 0;
}

/**
Evaluates a SymbolicFunction of one variable.  
@return double
*/
public double evaluate(double x)
{
    evaluationData[0] = x;
    return evaluate();  
}
/**
Evaluates a SymbolicFunction of two variables.
@return double
*/
public double evaluate(double x1, double x2)
{
    evaluationData[0] = x1;
    evaluationData[1] = x2;
    return evaluate();
};
/**
Evaluates a SymbolicFunctio of three variables.
@return double
*/
public double evaluate(double x1, double x2, double x3)
{
    evaluationData[0] = x1;
    evaluationData[1] = x2;
    evaluationData[2] = x3;
    return evaluate();  
};
/**
Evaluates a SymbolicFunctio of four variables.
@return double
*/
public double evaluate(double x1, double x2, double x3, double x4)
{
    evaluationData[0] = x1;
    evaluationData[1] = x2;
    evaluationData[2] = x3;
    evaluationData[3] = x4;
    return evaluate();  
};
/**
Evaluates a SymbolicFunction of n variables.
@param x double array of size n containing values for evaluation.
@return double
*/
public double evaluate(double [] x)
{
    int i;
    for(i = 0; i < x.length; i++) evaluationData[i] = x[i];
    return evaluate();
};
/**
Returns the string used to specify the function.
*/
public String  getConstructorString()
{
    return constructorString;
};
/**
Returns the number of independent variables in the function.
*/
public int     getVariableCount()
{
    return variableNames.length;
};
/**
Returns the name of the ith independent variable in the function. 
Indexing starts at 0.
*/
public String  getVariableName(int i)
{
    return new String(variableNames[i]);
}
/**
Returns the number of symbolic constants in the function. 
*/
public int     getConstantCount()
{
    return constantNames.length;
};
/**
Returns the name of the ith symbolic constant in the function. 
Indexing starts at 0.
*/
public String  getConstantName(int i)
{
    return new String(constantNames[i]);
}
/**
Returns the value of the ith symbolic constant in the function. 
Indexing starts at 0.
*/
public double  getConstantValue(int i)
{
    return constantValues[i];
}
/**
Sets the value of the symbolic constant in the function. 
Indexing starts at 0.
param S String name of the symbolic constant
param x double value for the symbolic constant
*/
public void  setConstantValue(String S,double x)
{
    int i;
    if(constantNames != null)
    {
    for(i = 0; i < constantNames.length; i++)
    {
    if(constantNames[i].equals(S)) constantValues[i] = x;
    }}  
    
    setConstantEvaluationData();
}

private double evaluate()
{
 	int j;
    int[] argIndex = new int[10];   // limit of 10 args for now

    int functionIndex;
    int argCount;

    int executionIndex = 0;
    int executionArraySize = executionArray.length;
    int evaluationDataSize = evaluationData.length;
    
    while(executionIndex < executionArraySize)
    {
    functionIndex = executionArray[executionIndex]; executionIndex++;
    argCount      = executionArray[executionIndex]; executionIndex++;
    for(j =0; j < argCount; j++)
    {
    argIndex[j] = executionArray[executionIndex];
    executionIndex++;
    }
    opLib.evaluate(functionIndex, argIndex, evaluationData);
    }

    return evaluationData[evaluationDataSize - 1];
}  

private void  setConstantEvaluationData()
{
	 int i; int j;

	 for(i = Vcount,j = 0; i < Vcount + Ccount; i++,j++)
	 evaluationData[i] = constantValues[j];
}
//
//   Internal Data 
//
private String     constructorString  = null;
private String[]       variableNames  = null;
private int            Vcount         = 0;

private String[]       constantNames  = null;
private double[]      constantValues  = null;
private int            Ccount         = 0;

private int []         executionArray = null;
private double []      evaluationData = null;

RealOperatorLib opLib;
//
//######################################################################
//  Main (For Testing)
//######################################################################
//
public static void main(String[] args)
{
    SymbolicFunction Sym = new SymbolicFunction(); 
    int i;


    String[] V = new String[1];                 
    V[0]       = new String("x");
    
    //String[] C = new String[1];                 
    //C[0]       = new String("a");

    String E = "-x^2";

    try
    {  
    Sym.initialize(V,E);
    double val = Sym.evaluate(2.0);
    System.out.println(" F = " + Sym.getConstructorString());
    System.out.println(" Evaluated at x = 2.0 is : " + val);
    }
    catch(Exception e)
    {
    System.out.println(e.getMessage());
    }
    
    System.out.println(" ");
    System.out.println(" Program End : Hit Return to Continue ");
    try{System.in.read();}catch(Exception e){};
}

}